@@ -19,7 +19,8 @@
 import toml
 import os
 import time
-import subprocess as sp
+import asyncio
+import textwrap
 from pathlib import Path
 from typing import Dict, List
 from .dep_manager import DepManager
@@ -28,12 +29,22 @@ from .exceptions import *
 
 
 class Build:
-    """Represents a build (see concepts)."""
+    """Represents a build (see concepts).
+    build_dir: Directory of the build root
+    warner: ZCBE warner
+    if_silent: whether to silence make stdout
+    """
 
-    def __init__(self, build_dir: str, warner: ZCBEWarner):
+    def __init__(
+        self,
+        build_dir: str,
+        warner: ZCBEWarner,
+        if_silent: bool = False
+    ):
         self.build_dir = Path(build_dir).absolute()
         self.build_toml = self.build_dir / "build.toml"
         self.warner = warner
+        self.if_silent = if_silent
         if self.build_toml.exists():
             self.parse_build_toml()
         else:
@@ -50,11 +61,15 @@ class Build:
         bdict = toml.load(self.build_toml)
         info = bdict["info"]
         try:
+            # Read configuration parameters
             self.build_name = info["build-name"]
             self.prefix = info["prefix"]
+            self.host = info["hostname"]
+            # Make sure prefix exists and is a directory
+            Path(self.prefix).mkdir(parents=True, exist_ok=True)
+            # Initialize dependency and built recorder
             self.dep_manager = DepManager(self.prefix+"/zcbe.recipe")
             os.environ["ZCPREF"] = self.prefix
-            self.host = info["hostname"]
             os.environ["ZCHOST"] = self.host
             os.environ["ZCTOP"] = self.build_dir.as_posix()
         except KeyError as e:
@@ -65,7 +80,7 @@ class Build:
 
     def get_proj_path(self, proj_name: str):
         """Get a project's root directory by looking up the mapping toml.
-        projname: The name of the projet to look up
+        projname: The name of the project to look up
         """
         self.mapping_toml = self.build_dir / "mapping.toml"
         if not self.mapping_toml.exists():
@@ -74,46 +89,68 @@ class Build:
         try:
             return self.build_dir / mapping[proj_name]
         except KeyError as e:
-            raise MappingTOMLError(f"project {proj_name} not found") from e
+            raise MappingTOMLError(f'project "{proj_name}" not found') from e
 
     def get_proj(self, proj_name: str):
         """Returns a project instance.
         projname: The name of the project
         """
         proj_path = self.get_proj_path(proj_name)
-        return Project(proj_path, proj_name, self, self.warner)
+        return Project(proj_path, proj_name, self)
 
-    def build(self, proj_name: str):
+    async def build(self, proj_name: str):
         """Build a project.
         proj_name: the name of the project
         """
+        # This project has already been built
         if self.dep_manager.check("req", proj_name):
             print(f"Requirement already satisfied: {proj_name}")
-            return 0
+            return
         proj = self.get_proj(proj_name)
-        proj.build()
+        # Circular dependency is handled by Python's recursion limitation
+        try:
+            await proj.build()
+        except RecursionError as e:
+            e.args = (
+                f'Circular dependency found near "{proj_name}"',) + e.args[1:]
+            raise
+
+    async def build_many(self, projs: List[str]) -> bool:
+        """Asynchronously build many projects.
+        projs: List of project names to be built
+        Returns whether the operations didn't raise anything.
+        """
+        successful = True
+        results = await asyncio.gather(
+            *(self.build(item) for item in projs), return_exceptions=True)
+        for idx, result in enumerate(results):
+            if result is not None:
+                successful = False
+                eprint(f'Project "{projs[idx]}" raised an exception:')
+                eprint(f"{type(result).__name__}: {result}", title=None)
+        return successful
 
 
 class Project:
     """Represents a project (see concepts).
     proj_dir is the directory to the project
     proj_name is the name in mapping.toml of the project
-    builder is used to resolve dependencies
-    warner is the ZCBEWarner to be used
+    builder is used to resolve dependencies, get warner and get if_silent
     """
 
     def __init__(self,
                  proj_dir: str,
                  proj_name: str,
-                 builder: Build,
-                 warner: ZCBEWarner
+                 builder: Build
                  ):
         self.proj_dir = Path(proj_dir)
         if not self.proj_dir.is_dir():
-            raise MappingTOMLError(f"project {proj_name} not found at {proj_dir}")
+            raise MappingTOMLError(
+                f"project {proj_name} not found at {proj_dir}")
         self.proj_name = proj_name
         self.builder = builder
-        self.warner = warner
+        self.warner = builder.warner
+        self.if_silent = builder.if_silent
         self.environ = os.environ
         self.conf_toml = self.locate_conf_toml()
         if self.conf_toml.exists():
@@ -142,16 +179,15 @@ class Project:
             return local_try
         raise ProjectTOMLError("conf.toml not found")
 
-    def solve_deps(self, depdict: Dict[str, List[str]]):
+    async def solve_deps(self, depdict: Dict[str, List[str]]):
         """Solve dependencies."""
         for table in depdict:
             if table == "build":
                 for item in depdict[table]:
                     self.builder.dep_manager.check(table, item)
             else:
-                for item in depdict[table]:
-                    # TODO: circular dep. check
-                    self.builder.build(item)
+                if not await self.builder.build_many(depdict[table]):
+                    raise BuildError("Dependency failed to build, stopping.")
 
     def parse_conf_toml(self):
         """Load the conf toml and set envs."""
@@ -162,7 +198,7 @@ class Project:
             if self.package_name != self.proj_name:
                 self.warner.warn(
                     "name-mismatch",
-                    "{self.package_name} mismatches with {self.proj_name}"
+                    f'"{self.package_name}" mismatches with "{self.proj_name}"'
                 )
             self.version = pkg["ver"]
         except KeyError as e:
@@ -176,45 +212,53 @@ class Project:
         else:
             self.envdict = {}
 
-    def acquire_lock(self):
+    async def acquire_lock(self):
         """Acquires project build lock."""
         lockfile = self.proj_dir / "zcbe.lock"
         while lockfile.exists():
-            print(f"The lockfile for project {self.proj_name} exists.")
-            print(
-                "If you're running multiple builds at the same time,",
-                "don't worry and we'll automatically proceed."
-            )
-            print(
-                "Otherwise please kill this process and remove the lock",
-                lockfile,
-                "by yourself. After that, check if everything is OK."
-            )
+            message = (f"The lockfile for project {self.proj_name} exists. "
+                       "If you're running multiple builds at the same time, "
+                       "don't worry and we'll automatically proceed. "
+                       "Otherwise please kill this process and remove "
+                       f'the lock file "{lockfile}" '
+                       "by yourself. After that, check if everything is OK.")
+            eprint('\n'.join(textwrap.wrap(message, 75)))
             time.sleep(10)
         lockfile.touch()
 
-    def release_lock(self):
+    async def release_lock(self):
         """Releases project build lock."""
         lockfile = self.proj_dir / "zcbe.lock"
         if lockfile.exists():
             lockfile.unlink()
 
-    def build(self):
+    async def build(self):
         """Solve dependencies and build the project."""
-        self.solve_deps(self.depdict)
+        await self.solve_deps(self.depdict)
         # Not infecting environ of other projects
         for item in self.envdict:
             self.environ[item[0]] = item[1]
-        self.acquire_lock()
+        await self.acquire_lock()
         print(f"Entering project {self.proj_name}")
         buildsh = self.locate_conf_toml().parent / "build.sh"
+        shpath = buildsh.as_posix()
         os.chdir(self.proj_dir)
-        complete = sp.run(["sh", "-e", buildsh], env=self.environ)
+        process = await asyncio.create_subprocess_exec(
+            "sh",
+            "-e",
+            shpath,
+            stdout=asyncio.subprocess.DEVNULL if self.if_silent else None,
+            env=self.environ,
+        )
+        await process.wait()
         print(f"Leaving project {self.proj_name}")
-        self.release_lock()
-        if complete.returncode:
+        await self.release_lock()
+        if process.returncode:
             # Build failed
             # Lock is still released as no one is writing to that directory
-            raise sp.CalledProcessError(complete.returncode, complete.args)
+            raise BuildError(
+                f"Command 'sh -e {shpath}' returned non-zero exit status"
+                f"{process.returncode}."
+            )
         # write recipe
         self.builder.dep_manager.add("req", self.proj_name)
