@@ -14,7 +14,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""ZCBE exceptions."""
+"""ZCBE exceptions and error handling."""
+import sys
+
+
+class BuildError(Exception):
+    pass
 
 
 class ConfigError(Exception):
@@ -31,3 +36,10 @@ class MappingTOMLError(ConfigError):
 
 class ProjectTOMLError(ConfigError):
     pass
+
+
+def eprint(*args, title="zcbe: *** Error: ", **kwargs):
+    """Print to stderr, use title as the starting if supplied."""
+    if title:
+        print(title, end='', file=sys.stderr)
+    print(*args, file=sys.stderr, **kwargs)
